body {
  margin: 10px;
  /* font-family: 'PT Sans', sans-serif; */
  padding-top: 3rem;
  padding-bottom: 3rem;
/*  color: #5a5a5a;*/
/*        font-size: 1.5vw;*/
  /* background-color:rgb(90, 90, 90); */
  background-color: black;
  background-color: #272822;
}

/* ul {
  padding-left: 12px;
}

ul li ul {
  display: none;
  padding-left: 12px;
}
*/

.sidebar ul {
  list-style: none;
  list-style-type: none;
  padding-left: 0.4em;
} 

header input {
  max-height: 30px;
  padding-top: 5px;
  margin-top: 10px;
}

.sidebar {
  /* background-color:rgb(90, 90, 90); */
  /* background-color: black; */
  background-color: #272822;
  color: white;
  font-size: 13px;
}

.sidebar a {
  color: white;
}

.col-md-10 a {
  color: white;
}

.col-md-9 a {
  color: white;
}

.nav-narrow {
  max-height: 20;
  padding-top: 0px;
  padding-bottom: 0px;
}

/* Folder styles */
body {
  font-family: Menlo, Consolas, monospace;
  /* color: #444; */
  color: white;
}
.item {
  cursor: pointer;
}
.bold {
  font-weight: bold;
}
ul {
  padding-left: 0.5em;
  line-height: 1.5em;
  list-style-type: dot;
}

/* Main Level */
ul{
  margin-left:1px;
}

/* Second Level */
ul ul{
  margin-left:1px;
}

/* Third Level */
ul ul ul{
  margin-left:1px;
}
